//  NearbyFriend
//  Copyright 2008 Urs Hengartner <uhengart@cs.uwaterloo.ca>
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of version 2 of the GNU General Public License as
//  published by the Free Software Foundation.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  There is a copy of the GNU General Public License in the COPYING file
//  packaged with this plugin; if you cannot find it, write to the Free
//  Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
//  02111-1307  USA

#ifndef __CGS97_H__
#define __CGS97_H__

#define DH1536_GROUP_ID 5

typedef struct {
    unsigned int groupid;
    gcry_mpi_t priv, pub;
} CGS97_Keypair;

typedef struct {
    gcry_mpi_t c1, c2;
} CGS97_Ciphertext;

/*
 * Call this once, at plugin load time.  It sets up the modulus and
 * generator MPIs.
 */
void nbf_cgs97_init(void);

/*
 * Deallocate the contents of a CGS97_Keypair (but not the CGS97_Keypair
 * itself)
 */
void nbf_cgs97_keypair_free(CGS97_Keypair *kp);

/*
 * Generate a CGS97 keypair for a specified group.
 */
gcry_error_t nbf_cgs97_gen_keypair(unsigned int groupid, CGS97_Keypair *kp);

/*
 * Encrypt message with CGS97.
 */
gcry_error_t nbf_cgs97_encrypt(gcry_mpi_t m, gcry_mpi_t pub,
			       CGS97_Ciphertext *ct);

/*
 * Set *answer = 1 if ciphertext decrypts to zero.
 */
gcry_error_t nbf_cgs97_decrypts_to_zero(CGS97_Ciphertext *ct, 
					CGS97_Keypair *kp, int *answer);

/*
 * result = ct1 + ct2;
 */
gcry_error_t nbf_cgs97_add(CGS97_Ciphertext *ct1, CGS97_Ciphertext *ct2,
			   CGS97_Ciphertext *result);

/*
 * result = ct1 - ct2
 */
gcry_error_t nbf_cgs97_subtract(CGS97_Ciphertext *ct1, CGS97_Ciphertext *ct2,
				CGS97_Ciphertext *result);

/*
 * result = ct - 1
 */
gcry_error_t nbf_cgs97_subtractOne(CGS97_Ciphertext *ct, gcry_mpi_t pub,
				   CGS97_Ciphertext *result);

gcry_error_t nbf_cgs97_multiply(CGS97_Ciphertext *ct, unsigned long k,
				CGS97_Ciphertext *result);

/*
 *  result =  ciphertext * random value
 */
gcry_error_t nbf_cgs97_multiply_random(CGS97_Ciphertext *ct, 
				       CGS97_Ciphertext *result);

/*
 * Initialize ciphertext
 */
gcry_error_t nbf_cgs97_ciphertext_init(CGS97_Ciphertext *ct, 
				       gcry_mpi_t c1, gcry_mpi_t c2);

/*
 * Deallocate the contents of a CGS97_Ciphertext (but not the 
 * CGS97_Ciphertext itself)
 */
gcry_error_t nbf_cgs97_ciphertext_free(CGS97_Ciphertext *ct);

/*
 * Return length of serialized ciphertext
 */

size_t nbf_cgs97_ciphertext_len_serialized(CGS97_Ciphertext *ct);
/*
 * Serialize ciphertext into (allocated) buffer
 */
void nbf_cgs97_ciphertext_serialize(CGS97_Ciphertext *ct, unsigned char* buf,
				    int buflen);

#endif
