//  NearbyFriend
//  Copyright 2008 Urs Hengartner <uhengart@cs.uwaterloo.ca>
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of version 2 of the GNU General Public License as
//  published by the Free Software Foundation.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  There is a copy of the GNU General Public License in the COPYING file
//  packaged with this plugin; if you cannot find it, write to the Free
//  Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
//  02111-1307  USA

#ifndef __CONTEXT_H__
#define __CONTEXT_H__

/* libgcrypt headers */
#include <gcrypt.h>

/* nbf headers */
#include "cgs97.h"

typedef enum {
    NBF_CONV_STATE_INIT,           /* initial state */ 
    NBF_CONV_STATE_SENT_PING,      /* sent ping, waiting for pong */
    NBF_CONV_STATE_GOT_PING,       /* got ping, we'll send pong as soon as we can */
    NBF_CONV_STATE_READY           /* ready to run NearbyFriend protocol */
} NbfConversationState;

typedef enum {               
    NBF_ALICE_STATE_READY,         /* ready to start new run of the protocol */
    NBF_ALICE_STATE_WAIT_RESPONSE  /* sent query, waiting for response */
} NbfProtocolStateAlice;

/* 
 * Each IM conversation has a context associated with it, regardless
 * whether it runs NearbyFriend or not.
 */  
typedef struct context {
    struct context * next;             /* Linked list pointer */
    struct context ** tous;            /* A pointer to the pointer to us */

    char * username;                   /* The user this context is for */
    char * accountname;                /* The username is relative to
					  this account... */
    char * protocol;                   /* ... and this protocol */

    NbfConversationState conv_state;   /* The state of the conversation */
    NbfProtocolStateAlice alice_prot_state; /* The protocol state for the Alice side of the protocol */
    /* The Bob side simply sends a response whenever he gets a query */
    gcry_mpi_t peer_y;                 /* our peer's CGS97 public key */    

} NbfConnContext;

#include "userstate.h"

/* Look up a connection context by name/account/protocol from the given
 * NbfUserState.  If add_if_missing is true, allocate and return a new
 * context if one does not currently exist.  In that event, call
 * add_app_data(data, context) so that app_data and app_data_free can be
 * filled in by the application, and set *addedp to 1. */
NbfConnContext * nbf_context_find(NbfUserState* us, const char *user,
	const char *accountname, const char *protocol, int add_if_missing,
				  int *addedp);

#endif
