//  NearbyFriend
//  Copyright 2008 Urs Hengartner <uhengart@cs.uwaterloo.ca>
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of version 2 of the GNU General Public License as
//  published by the Free Software Foundation.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  There is a copy of the GNU General Public License in the COPYING file
//  packaged with this plugin; if you cannot find it, write to the Free
//  Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
//  02111-1307  USA

/* Modified from: */

/*
 *  Off-the-Record Messaging plugin for pidgin
 *  Copyright (C) 2004-2007  Ian Goldberg, Chris Alexander, Nikita Borisov
 *                           <otr@cypherpunks.ca>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of version 2 of the GNU General Public License as
 *  published by the Free Software Foundation.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/* system headers */
#include <stdlib.h>

/* This will prevent compiler errors in some instances and is better explained in the
 * how-to documents on the wiki */
#ifndef G_GNUC_NULL_TERMINATED
# if __GNUC__ >= 4
#  define G_GNUC_NULL_TERMINATED __attribute__((__sentinel__))
# else
#  define G_GNUC_NULL_TERMINATED
# endif
#endif

/* pidgin headers */
#include "notify.h"

/* nbf headers */
#include "dialogs.h"

static const NbfDialogUiOps *ui_ops = NULL;

/* Set the UI ops */
void nbf_dialog_set_ui_ops(const NbfDialogUiOps *ops)
{
    ui_ops = ops;
}

static void nbf_dialog_notify_message(PurpleNotifyMsgType type,
        const char *accountname, const char *protocol, const char *username,
        const char *title, const char *primary, const char *secondary)
{
    ui_ops->notify_message(type, accountname, protocol, username,
            title, primary, secondary);
}

/* Put up the error version of nbf_dialog_notify_message */
void nbf_dialog_notify_error(const char *accountname, const char *protocol,
        const char *username, const char *title, const char *primary,
        const char *secondary)
{
    nbf_dialog_notify_message(PURPLE_NOTIFY_MSG_ERROR, accountname,
            protocol, username, title, primary, secondary);
}

/* Display an NBF control message for the given accountname / protocol /
 * username conversation.  Return 0 on success, non-0 on error (in which
 * case the message will be displayed inline as a received message). */
int nbf_dialog_display_nbf_message(const char *accountname,
        const char *protocol, const char *username, const char *msg)
{
    return ui_ops->display_nbf_message(accountname, protocol, username, msg);
}
