//  NearbyFriend
//  Copyright 2008 Urs Hengartner <uhengart@cs.uwaterloo.ca>
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of version 2 of the GNU General Public License as
//  published by the Free Software Foundation.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  There is a copy of the GNU General Public License in the COPYING file
//  packaged with this plugin; if you cannot find it, write to the Free
//  Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
//  02111-1307  USA
 
#include "config.h"

/* system headers */
#include <gtk/gtk.h>

/* This will prevent compiler errors in some instances and is better explained in the
 * how-to documents on the wiki */
#ifndef G_GNUC_NULL_TERMINATED
# if __GNUC__ >= 4
#  define G_GNUC_NULL_TERMINATED __attribute__((__sentinel__))
# else
#  define G_GNUC_NULL_TERMINATED
# endif
#endif

/* libgcrypt headers */
#include <gcrypt.h>

/* purple headers */
#include "util.h"
#include "account.h"
#include "notify.h"
#include "gtkutils.h"

#ifdef ENABLE_NLS
/* internationalisation header */
#include <glib/gi18n-lib.h>
#endif

/* purple-nbf headers */
#include "dialogs.h"
#include "ui.h"
#include "nbf-plugin.h"
#include "gtk-ui.h"

struct nbfsettingsdata {
  // these two can now only be changed directly in the configuration file
  //GtkWidget *samplingfrequency;
  //GtkWidget *locationstale;
  GtkWidget *nearbydistance;
  GtkWidget *queryfrequency;
};

static struct {
  GtkWidget *accountmenu;
  GtkWidget *fprint_label;
  
  GtkWidget *scrollwin;
  GtkWidget *buddylist;
  gint sortcol, sortdir;
  
  GtkWidget *connect_all_button;
  GtkWidget *connect_button;
  GtkWidget *disconnect_button;
  GtkWidget *disconnect_all_button;
  PurpleBlistNode * selected_buddy;
  struct nbfsettingsdata os;
} ui_layout;

static void clist_all_unselected(void)
{
  gtk_widget_set_sensitive(ui_layout.connect_button, 0);
  gtk_widget_set_sensitive(ui_layout.disconnect_button, 0);
  ui_layout.selected_buddy = NULL;
}

static void ui_destroyed(GtkObject *object)
{
  /* If this is called, we need to invalidate the stored pointers in
   * the ui_layout struct. */
  ui_layout.accountmenu = NULL;
  ui_layout.fprint_label = NULL;
  ui_layout.scrollwin = NULL;
  ui_layout.buddylist = NULL;
  ui_layout.connect_button = NULL;
  ui_layout.disconnect_button = NULL;
  
  //ui_layout.os.samplingfrequency = NULL;
  //ui_layout.os.locationstale = NULL;
  ui_layout.os.nearbydistance = NULL;
  ui_layout.os.queryfrequency = NULL;
}

static gint statuscmp(GtkCList *clist, gconstpointer ptr1, gconstpointer ptr2)
{
  return 0;
}

static void clist_click_column(GtkCList *clist, gint column, gpointer data)
{
  if (ui_layout.sortcol == column) {
	ui_layout.sortdir = -(ui_layout.sortdir);
  } else {
	ui_layout.sortcol = column;
	ui_layout.sortdir = 1;
  }
  
  gtk_clist_set_sort_column(clist, ui_layout.sortcol);
  gtk_clist_set_sort_type(clist,
			  ui_layout.sortdir == 1 ? GTK_SORT_ASCENDING : GTK_SORT_DESCENDING);
  if (column == 1) {
	gtk_clist_set_compare_func(clist, statuscmp);
  } else {
	/* Just use the default compare function for the rest of the
	 * columns */
	gtk_clist_set_compare_func(clist, NULL);
  }
  gtk_clist_sort(clist);
}

static void create_nbfsettings_entries(struct nbfsettingsdata *os, GtkWidget *vbox) {
  GtkTable *tbox;  
  GtkWidget *label1, *label2;

  /*
   os->samplingfrequency = gtk_entry_new_with_max_length (10);
   os->locationstale = gtk_entry_new_with_max_length (10);
   */
  os->nearbydistance = gtk_entry_new_with_max_length (10);
  os->queryfrequency = gtk_entry_new_with_max_length (10);
  
  tbox = (GtkTable*)gtk_table_new(2,2,TRUE);
  gtk_table_set_row_spacing(tbox, 0, 10);
  label1 = gtk_label_new("Maximum distance that is still \n"
			 "considered nearby (in meters)");
  label2 = gtk_label_new("Minimum time interval between two \n"
			 "queries (in seconds)");

  gtk_misc_set_alignment(GTK_MISC(label1),0.0,0.5);
  gtk_misc_set_alignment(GTK_MISC(label2),0.0,0.5);
  /*
   gtk_table_attach_defaults(tbox, gtk_label_new("Sampling Frequency:"),0,1,0,1);
   gtk_table_attach_defaults(tbox, os->samplingfrequency,1,2,0,1);
   gtk_table_attach_defaults(tbox, gtk_label_new("Location Stale:"),0,1,1,2);
   gtk_table_attach_defaults(tbox, os->locationstale,1,2,1,2);
   */
  gtk_table_attach_defaults(tbox, label1,0,1,0,1);
  gtk_table_attach_defaults(tbox, os->nearbydistance,1,2,0,1);
  gtk_table_attach_defaults(tbox, label2,0,1,1,2);
  gtk_table_attach_defaults(tbox, os->queryfrequency,1,2,1,2);
  
  gtk_box_pack_start(GTK_BOX(vbox), (GtkWidget*)tbox, FALSE, FALSE, 0);
  
}

/* Load the global NBF prefs */
static void nbfg_gtk_ui_global_prefs_load(gint *samplingfrequency,
										  gint *locationstale, gint *nearbydistance,
										  gint *queryfrequency)
{
  gint sf = 0;
  gint ls = 0;
  gint nbd = 0;
  gint qf = 0;
  if (purple_prefs_exists("/NBF/samplingfrequency")) {
	
	sf = purple_prefs_get_int("/NBF/samplingfrequency");
	ls = purple_prefs_get_int("/NBF/locationstale");
	nbd = purple_prefs_get_int("/NBF/nearbydistance");
	qf = purple_prefs_get_int("/NBF/queryfrequency");
	
	*samplingfrequency = sf;
	*locationstale = ls;
	*nearbydistance = nbd;
	*queryfrequency = qf;
  } else {
	*samplingfrequency = SAMPLING_FREQUENCY;
	*locationstale = LOCATION_STALE;
	*nearbydistance = NEARBY_DISTANCE;
	*queryfrequency = QUERY_FREQUENCY;
  }
}

/* Save the global NBF prefs */
static void nbfg_gtk_ui_global_prefs_save(gint samplingfrequency,
					  gint locationstale, 
					  gint nearbydistance, 
					  gint queryfrequency)
{
  if (! purple_prefs_exists("/NBF")) {
	purple_prefs_add_none("/NBF");
	purple_prefs_add_int("/NBF/samplingfrequency", samplingfrequency);
	purple_prefs_add_int("/NBF/locationstale", locationstale);
	purple_prefs_add_int("/NBF/nearbydistance", nearbydistance);
	purple_prefs_add_int("/NBF/queryfrequency", queryfrequency);
  } else {
	purple_prefs_set_int("/NBF/samplingfrequency", samplingfrequency);
	purple_prefs_set_int("/NBF/locationstale", locationstale);
	purple_prefs_set_int("/NBF/nearbydistance", nearbydistance);
	purple_prefs_set_int("/NBF/queryfrequency", queryfrequency);
  }
}

#if 0
/* Load the NBF prefs for a particular buddy */
static void nbfg_gtk_ui_buddy_prefs_load(PurpleBuddy *buddy, gboolean *enabledp)
{
  PurpleBlistNode *node = &(buddy->node);
  *enabledp = purple_blist_node_get_bool(node, "NBF/enabled");
}
#endif

static void load_nbfsettings(struct nbfsettingsdata *os)
{
  
  gint nbfsamplingfrequency;
  gint nbflocationstale;
  gint nbfnearbydistance;
  gint nbfqueryfrequency;
  gchar buffer[10];
  
  nbfg_gtk_ui_global_prefs_load(&nbfsamplingfrequency, 
				&nbflocationstale, 
				&nbfnearbydistance,
				&nbfqueryfrequency);

  //sprintf (buffer, "%d", nbfsamplingfrequency);
  //gtk_entry_set_text(GTK_ENTRY(os->samplingfrequency),
  //				 buffer);
  //sprintf (buffer, "%d", nbflocationstale);
  //gtk_entry_set_text(GTK_ENTRY(os->locationstale),
  //					 buffer);
  sprintf (buffer, "%d", nbfnearbydistance);
  gtk_entry_set_text(GTK_ENTRY(os->nearbydistance),
					 buffer);
  sprintf (buffer, "%d", nbfqueryfrequency);
  gtk_entry_set_text(GTK_ENTRY(os->queryfrequency),
					 buffer);
  
}

/* Save the global NBF settings whenever they're clicked */
static void nbfsettings_save_cb(GtkButton *button, struct nbfsettingsdata *os)
{
  gint samplingfrequency, locationstale, nearbydistance, queryfrequency;

  // load current values from config file and overwrite the ones that 
  // can be changed in GUI

  nbfg_gtk_ui_global_prefs_load(&samplingfrequency, 
				&locationstale, 
				&nearbydistance,
				&queryfrequency);
  
  //samplingfrequency = atoi((gchar *) gtk_entry_get_text (GTK_ENTRY (ui_layout.os.samplingfrequency)));
  //locationstale = atoi((gchar *) gtk_entry_get_text (GTK_ENTRY (ui_layout.os.locationstale)));
  nearbydistance = atoi((gchar *) gtk_entry_get_text (GTK_ENTRY (ui_layout.os.nearbydistance)));
  queryfrequency = atoi((gchar *) gtk_entry_get_text (GTK_ENTRY (ui_layout.os.queryfrequency)));
  nbfg_gtk_ui_global_prefs_save(samplingfrequency, locationstale, nearbydistance, queryfrequency);
  
}

/* Make the settings UI, and pack it into the vbox */
static void make_settings_ui(GtkWidget *vbox)
{
  
  GtkWidget *fbox;
  GtkWidget *frame;
  
  frame = gtk_frame_new("NearbyFriend Settings");
  gtk_box_pack_start(GTK_BOX(vbox), frame, FALSE, FALSE, 0);
  
  fbox = gtk_vbox_new(FALSE, 0);
  gtk_container_set_border_width(GTK_CONTAINER(fbox), 10);
  gtk_container_add(GTK_CONTAINER(frame), fbox);
  
  create_nbfsettings_entries(&(ui_layout.os), fbox);
  
  load_nbfsettings(&(ui_layout.os));
  
  g_signal_connect(G_OBJECT(ui_layout.os.nearbydistance), "changed",
				   G_CALLBACK(nbfsettings_save_cb), &(ui_layout.os));
  g_signal_connect(G_OBJECT(ui_layout.os.queryfrequency), "changed",
				   G_CALLBACK(nbfsettings_save_cb), &(ui_layout.os));
  
}

/* Update the buddylist, if it's visible */
static void nbfg_gtk_ui_update_buddylist(void)
{
  GtkWidget *buddylist = ui_layout.buddylist;
  PurpleBlistNode *buddy_node = purple_blist_get_root();
  gchar *titles[3];
  
  gint selected_row = -1;
  gint i;
  if (buddylist == NULL)
	return;
  
  gtk_clist_freeze(GTK_CLIST(buddylist));
  gtk_clist_clear(GTK_CLIST(buddylist));
  
  while (buddy_node)
  {
	if (PURPLE_BLIST_NODE_IS_BUDDY (buddy_node)) {
	  const PurpleBuddy * buddy = (PurpleBuddy*) (buddy_node);
	  PurplePresence * pres = purple_buddy_get_presence (buddy);
	  PurpleStatus * status = purple_presence_get_active_status (pres);
	  
	  titles[0] = buddy->name;
	  titles[1] = (gchar*)purple_status_get_name (status);
	  if (purple_blist_node_get_bool(buddy_node, "NBF/enabled")) {
		titles[2] = "yes";
	  } else {
		titles[2] = "no";
	  }
	  i = gtk_clist_append(GTK_CLIST(buddylist), titles);
	  gtk_clist_set_row_data(GTK_CLIST(buddylist), i, buddy_node);
	  if (buddy_node == ui_layout.selected_buddy) {
		selected_row = i;
	  }
	}
	buddy_node = purple_blist_node_next (buddy_node, TRUE);
  }
  
  if (selected_row >= 0) {
	gtk_clist_select_row(GTK_CLIST(buddylist), selected_row, 0);
  } else {
	clist_all_unselected();
  }
  
  gtk_clist_sort(GTK_CLIST(buddylist));
  
  gtk_clist_thaw(GTK_CLIST(buddylist));
  
}

static void clist_unselected(GtkWidget *widget, gint row, gint column,
							 GdkEventButton *event, gpointer data)
{
  clist_all_unselected();
}

static void clist_selected(GtkWidget *widget, gint row, gint column,
						   GdkEventButton *event, gpointer data)
{
  gboolean enable_sensitive = FALSE;
  ui_layout.selected_buddy = gtk_clist_get_row_data(GTK_CLIST(ui_layout.buddylist), row);
  if (purple_blist_node_get_bool(ui_layout.selected_buddy, "NBF/enabled")) {
	enable_sensitive = FALSE;
  } else {
	enable_sensitive = TRUE;
  }
  
  gtk_widget_set_sensitive(ui_layout.connect_button, enable_sensitive);
  gtk_widget_set_sensitive(ui_layout.disconnect_button, !enable_sensitive);
  
}

static void enable_nbf(GtkWidget *widget, gpointer data)
{
  if (ui_layout.selected_buddy == NULL) return;
  purple_blist_node_set_bool(ui_layout.selected_buddy, "NBF/enabled", TRUE);
  nbfg_gtk_ui_update_buddylist();
}

static void disable_nbf(GtkWidget *widget, gpointer data)
{
  if (ui_layout.selected_buddy == NULL) return;
  purple_blist_node_set_bool(ui_layout.selected_buddy, "NBF/enabled", FALSE);
  nbfg_gtk_ui_update_buddylist();
}

static void enable_all_nbf(GtkWidget *widget, gpointer data)
{
  PurpleBlistNode *buddy_node = purple_blist_get_root();
  
  while (buddy_node)
  {
	if (PURPLE_BLIST_NODE_IS_BUDDY (buddy_node)) {
	  purple_blist_node_set_bool(buddy_node, "NBF/enabled", TRUE);
	}
	buddy_node = purple_blist_node_next (buddy_node, TRUE);
  }
  nbfg_gtk_ui_update_buddylist();
}

static void disable_all_nbf(GtkWidget *widget, gpointer data)
{
  PurpleBlistNode *buddy_node = purple_blist_get_root();
  
  while (buddy_node)
  {
	if (PURPLE_BLIST_NODE_IS_BUDDY (buddy_node)) {
	  purple_blist_node_set_bool(buddy_node, "NBF/enabled", FALSE);
	}
	buddy_node = purple_blist_node_next (buddy_node, TRUE);
  }
  nbfg_gtk_ui_update_buddylist();
}

/* Create the buddy enable UI, and pack it into the vbox */
static void make_fingerprints_ui(GtkWidget *vbox)
{
  GtkWidget *hbox;
  GtkWidget *table;
  GtkWidget *label;
  char *titles[3];
  
  titles[0] = "Screenname";
  titles[1] = "Status";
  titles[2] = "Query whether nearby?";
  
  ui_layout.scrollwin = gtk_scrolled_window_new(0, 0);
  gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(ui_layout.scrollwin), 
								 GTK_POLICY_ALWAYS, GTK_POLICY_ALWAYS);
  
  ui_layout.buddylist = gtk_clist_new_with_titles(3, titles);
  gtk_clist_set_column_width(GTK_CLIST(ui_layout.buddylist), 0, 210);
  gtk_clist_set_column_width(GTK_CLIST(ui_layout.buddylist), 1, 70);
  gtk_clist_set_column_width(GTK_CLIST(ui_layout.buddylist), 2, 120);
  
  gtk_clist_set_selection_mode(GTK_CLIST(ui_layout.buddylist),
							   GTK_SELECTION_SINGLE);
  gtk_clist_column_titles_active(GTK_CLIST(ui_layout.buddylist));
  
  gtk_container_add(GTK_CONTAINER(ui_layout.scrollwin), ui_layout.buddylist);
  gtk_box_pack_start(GTK_BOX(vbox), ui_layout.scrollwin,
					 TRUE, TRUE, 0);
  
  nbfg_gtk_ui_update_buddylist();
  
  hbox = gtk_hbox_new(FALSE, 5);
  gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 5);
  
  table = gtk_table_new(1, 4, TRUE);
  gtk_table_set_row_spacings(GTK_TABLE(table), 5);
  gtk_table_set_col_spacings(GTK_TABLE(table), 20);
  
  gtk_box_pack_start(GTK_BOX(hbox), gtk_label_new(""), TRUE, TRUE, 0);
  gtk_box_pack_start(GTK_BOX(hbox), table, FALSE, FALSE, 0);
  gtk_box_pack_start(GTK_BOX(hbox), gtk_label_new(""), TRUE, TRUE, 0);
  
  ui_layout.connect_all_button = gtk_button_new();
  gtk_signal_connect(GTK_OBJECT(ui_layout.connect_all_button), "clicked",
					 GTK_SIGNAL_FUNC(enable_all_nbf), NULL);
  label = gtk_label_new("Enable all");
  gtk_container_add(GTK_CONTAINER(ui_layout.connect_all_button), label);
  gtk_table_attach_defaults(GTK_TABLE(table), ui_layout.connect_all_button,
							0, 1, 0, 1);
  
  ui_layout.connect_button = gtk_button_new();
  gtk_signal_connect(GTK_OBJECT(ui_layout.connect_button), "clicked",
					 GTK_SIGNAL_FUNC(enable_nbf), NULL);
  label = gtk_label_new("Enable selected");
  gtk_container_add(GTK_CONTAINER(ui_layout.connect_button), label);
  gtk_table_attach_defaults(GTK_TABLE(table), ui_layout.connect_button,
							1, 2, 0, 1);
  
  ui_layout.disconnect_button = gtk_button_new();
  gtk_signal_connect(GTK_OBJECT(ui_layout.disconnect_button), "clicked",
					 GTK_SIGNAL_FUNC(disable_nbf), NULL);
  label = gtk_label_new("Disable selected");
  gtk_container_add(GTK_CONTAINER(ui_layout.disconnect_button), label);
  gtk_table_attach_defaults(GTK_TABLE(table), ui_layout.disconnect_button,
							2, 3, 0, 1);
  
  ui_layout.disconnect_all_button = gtk_button_new();
  gtk_signal_connect(GTK_OBJECT(ui_layout.disconnect_all_button), "clicked",
					 GTK_SIGNAL_FUNC(disable_all_nbf), NULL);
  label = gtk_label_new("Disable all");
  gtk_container_add(GTK_CONTAINER(ui_layout.disconnect_all_button), label);
  gtk_table_attach_defaults(GTK_TABLE(table), ui_layout.disconnect_all_button,
							3, 4, 0, 1);
  
  gtk_signal_connect(GTK_OBJECT(vbox), "destroy",
					 GTK_SIGNAL_FUNC(ui_destroyed), NULL);
  
  /* Handle selections and deselections */
  gtk_signal_connect(GTK_OBJECT(ui_layout.buddylist), "select_row",
					 GTK_SIGNAL_FUNC(clist_selected), NULL);
  gtk_signal_connect(GTK_OBJECT(ui_layout.buddylist), "unselect_row",
					 GTK_SIGNAL_FUNC(clist_unselected), NULL);
  
  /* Handle column sorting */
  gtk_signal_connect(GTK_OBJECT(ui_layout.buddylist), "click-column",
					 GTK_SIGNAL_FUNC(clist_click_column), NULL);
  ui_layout.sortcol = 0;
  ui_layout.sortdir = 1;
  
  clist_all_unselected();
}


/* Construct the NBF UI widget */
GtkWidget* nbfg_gtk_ui_make_widget(PurplePlugin *plugin)
{
  GtkWidget *vbox = gtk_vbox_new(FALSE, 5);
  GtkWidget *fingerprintbox = gtk_vbox_new(FALSE, 5);
  GtkWidget *configbox = gtk_vbox_new(FALSE, 5);
  GtkWidget *notebook = gtk_notebook_new();
  
  gtk_container_set_border_width(GTK_CONTAINER(vbox), 2);
  gtk_container_set_border_width(GTK_CONTAINER(fingerprintbox), 5);
  gtk_container_set_border_width(GTK_CONTAINER(configbox), 5);
  
  gtk_box_pack_start(GTK_BOX(vbox), notebook, TRUE, TRUE, 0);

  make_settings_ui(configbox);

  make_fingerprints_ui(fingerprintbox);
  
  gtk_notebook_append_page(GTK_NOTEBOOK(notebook), configbox,
						   gtk_label_new("Config"));
  gtk_notebook_append_page(GTK_NOTEBOOK(notebook), fingerprintbox,
						   gtk_label_new("Buddy Config"));
  
  gtk_widget_show_all(vbox);
  return vbox;
  
}

struct cbdata {
  GtkWidget *dialog;
  PurpleBuddy *buddy;
  GtkWidget *defaultbox;
  struct nbfsettingsdata os;
};

static void nbfg_gtk_ui_config_buddy(PurpleBuddy *buddy)
{
}


/* Load the preferences for a particular account / username */
static void nbfg_gtk_ui_get_prefs(NbfgUiPrefs *prefsp)
{
  
  nbfg_gtk_ui_global_prefs_load(&(prefsp->samplingfrequency), 
				&(prefsp->locationstale), 
				&(prefsp->nearbydistance),
				&(prefsp->queryfrequency));
}

/* Initialize the NBF UI subsystem */
static void nbfg_gtk_ui_init(void)
{
  /* Nothing to do */
}

/* Deinitialize the NBF UI subsystem */
static void nbfg_gtk_ui_cleanup(void)
{
  /* Nothing to do */
}

static const NbfgUiUiOps gtk_ui_ui_ops = {
  nbfg_gtk_ui_init,
  nbfg_gtk_ui_cleanup,
  nbfg_gtk_ui_config_buddy,
  nbfg_gtk_ui_get_prefs
};

/* Get the GTK UI ops */
const NbfgUiUiOps *nbfg_gtk_ui_get_ui_ops(void)
{
  return &gtk_ui_ui_ops;
}
