//  NearbyFriend
//  Copyright 2008 Urs Hengartner <uhengart@cs.uwaterloo.ca>
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of version 2 of the GNU General Public License as
//  published by the Free Software Foundation.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  There is a copy of the GNU General Public License in the COPYING file
//  packaged with this plugin; if you cannot find it, write to the Free
//  Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
//  02111-1307  USA

#ifndef __NEARBYFRIEND_H__
#define __NEARBYFRIEND_H__

/* purple headers */
#include <conversation.h>

/* nbf headers */
#include "userstate.h"
#include "proto.h"

typedef struct s_NbfMessageAppOps {

    /* Send the given IM to the given recipient from the given
     * accountname/protocol. */
    void (*send_im)(const char *accountname,
            const char *protocol, const char *recipient, const char *message);

    /* Display an NBF control message for a particular accountname /
     * protocol / username conversation.  Return 0 if you are able to
     * successfully display it.  If you return non-0 (or if this
     * function is NULL), the control message will be displayed inline,
     * as a received message, or else by using the above notify()
     * callback. */
    int (*display_nbf_message)(const char *accountname,
            const char *protocol, const char *username, const char *msg);

} NbfMessageAppOps;

/*
 * Called once at beginning
 */
void nbf_init();

/*
 * Periodically invoked to send a query to all peers
 */
gboolean nbf_send_query(gpointer data);

/*
 * Process a received message, send a response if necessary. If it
 * returns 0, message was not for NearbyFriend. If it returns 1,
 * *newmessage might contain a new message, which should be shown to
 * user, instead of the received message. 
 */
int nbf_receive_message(NbfUserState* us, const NbfMessageAppOps *ops, 
			void *opdata, const char* accountname, 
			const char* protocol, const char* username, 
			const char *message);

/*
 * Invoked whenever a new conversation is set up
 */
void nbf_process_new_conversation(NbfUserState* us, 
				  const NbfMessageAppOps *ops,
				  PurpleConversation *conv);

/*
 * Invoked whenever a conversation is deleted
 */
void nbf_process_delete_conversation(NbfUserState* us, 
				     const NbfMessageAppOps *ops,
				     PurpleConversation *conv);

/*
 * Send a ping or a stop message to all IM conversations
 */
void nbf_send_msg_to_everybody(NbfMessageType type, NbfUserState* userstate, 
			       NbfMessageAppOps* ops);

#endif
