//  NearbyFriend
//  Copyright 2008 Urs Hengartner <uhengart@cs.uwaterloo.ca>
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of version 2 of the GNU General Public License as
//  published by the Free Software Foundation.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  There is a copy of the GNU General Public License in the COPYING file
//  packaged with this plugin; if you cannot find it, write to the Free
//  Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
//  02111-1307  USA

#ifndef __PROTO_H__
#define __PROTO_H__

/* libgcrypt headers */
#include <gcrypt.h>

/* nbf headers */
#include "userstate.h"
#include "context.h"
#include "cgs97.h"

typedef enum {
    NBF_MSGTYPE_NOTNBF,
    NBF_MSGTYPE_PING,
    NBF_MSGTYPE_PONG,
    NBF_MSGTYPE_QUERY,
    NBF_MSGTYPE_RESPONSE,
    NBF_MSGTYPE_STOP,
    NBF_MSGTYPE_UNKNOWN
} NbfMessageType;

NbfMessageType nbf_proto_message_type(const char *message);

gcry_error_t nbf_proto_create_ping(char **ping, const char* our_name);

gcry_error_t nbf_proto_create_pong(char **pong);

gcry_error_t nbf_proto_create_query(char** query,
				    gcry_mpi_t pub,
				    CGS97_Ciphertext* ctsquare,
				    CGS97_Ciphertext* ctx2,
				    CGS97_Ciphertext* cty2);

gcry_error_t nbf_proto_create_response(char **response,
				       CGS97_Ciphertext* ct1,
				       CGS97_Ciphertext* ct2,
				       CGS97_Ciphertext* ct3);		    

gcry_error_t nbf_proto_create_error_response(char **response);

gcry_error_t nbf_proto_create_stop(char **stop);

/*
 * Parse query message into public key and ciphertexts 
 */
gcry_error_t nbf_proto_accept_query(gcry_mpi_t* pub,
				    CGS97_Ciphertext* ctsquare, 
				    CGS97_Ciphertext* ctx2, 
				    CGS97_Ciphertext* cty2, 
				    NbfConnContext *context, 
				    const char *msg);

/*
 * Parse response message into ciphertexts 
 */
gcry_error_t nbf_proto_accept_response(CGS97_Ciphertext* ct1,
				       CGS97_Ciphertext* ct2,
				       CGS97_Ciphertext* ct3,
				       NbfUserState *userstate,
				       NbfConnContext *context, 
				       const char *msg);

/*
 * Return current time in milliseconds since the Epoch
 */
int nbf_get_current_time_millis(void);

/*
 * Initialize big integer with a 64 bit value
 */
gcry_mpi_t nbf_mpi_set_long_long(long long value);

#endif
