//  NearbyFriend
//  Copyright 2008 Urs Hengartner <uhengart@cs.uwaterloo.ca>
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of version 2 of the GNU General Public License as
//  published by the Free Software Foundation.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  There is a copy of the GNU General Public License in the COPYING file
//  packaged with this plugin; if you cannot find it, write to the Free
//  Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
//  02111-1307  USA

#include "config.h"

/* system headers */
#include <stdlib.h>

/* This will prevent compiler errors in some instances and is better explained in the
 * how-to documents on the wiki */
#ifndef G_GNUC_NULL_TERMINATED
# if __GNUC__ >= 4
#  define G_GNUC_NULL_TERMINATED __attribute__((__sentinel__))
# else
#  define G_GNUC_NULL_TERMINATED
# endif
#endif

/* purple headers */
#include "util.h"
#include "account.h"

#ifdef ENABLE_NLS
/* internationalisation header */
#include <glib/gi18n-lib.h>
#endif

/* purple-nbf headers */
#include "ui.h"
#include "dialogs.h"
#include "nbf-plugin.h"

static const NbfgUiUiOps *ui_ops = NULL;

/* Set the UI ops */
void nbfg_ui_set_ui_ops(const NbfgUiUiOps *ops)
{
  ui_ops = ops;
}

/* Get the UI ops */
const NbfgUiUiOps *nbfg_ui_get_ui_ops(void)
{
  return ui_ops;
}

/* Initialize the NBF UI subsystem */
void nbfg_ui_init(void)
{
  if (ui_ops != NULL) {
	ui_ops->init();
  }
}

/* Deinitialize the NBF UI subsystem */
void nbfg_ui_cleanup(void)
{
  if (ui_ops != NULL) {
	ui_ops->cleanup();
  }
}
/* Configure NBF for a particular buddy */
void nbfg_ui_config_buddy(PurpleBuddy *buddy)
{
  if (ui_ops != NULL) {
	ui_ops->config_buddy(buddy);
  }
}

/* Load the preferences for a particular account / username */
void nbfg_ui_get_prefs(NbfgUiPrefs *prefsp)
{
  if (ui_ops != NULL) {
      ui_ops->get_prefs(prefsp);
      return;
  }

  /* If we've got no other way to get the prefs, use sensible defaults */
  prefsp->samplingfrequency = SAMPLING_FREQUENCY;
  prefsp->locationstale = LOCATION_STALE;
  prefsp->nearbydistance = NEARBY_DISTANCE;
  prefsp->queryfrequency = QUERY_FREQUENCY;
}
